------------------------------------------------
-- Loot Drop - show me what I got
--
-- @author Pawkette ( pawkette.heals@gmail.com )
------------------------------------------------
local LootDrop_MaxVetRank        = 14
local LootDropPool      = LootDropPool
local LootDrop          = LootDropPool:Subclass()
LootDrop.dirty_flags    = setmetatable( {}, { __mode = 'kv'} )
LootDrop.config         = nil
LootDrop.db             = nil
LootDrop.ScreenMaxWidth = 800
LootDrop.ScreenMaxHeight = 600
LootDrop.CurrentItemBag_ItemStyle = nil
LootDrop.CurrentItemBag_ItemType = nil
LootDrop.CurrentItemBag_ItemLink = nil
LootDrop.CurrentItemBag_ItemQuality = nil
LootDrop.CurrentItemBag_ItemNb = nil
LootDrop.ItemToPrint={iconFileName=[[/esoui/art/icons/icon_missing.dds]], color='FFFFFF', quantity=0, nb=0, text='', tag='', itemstyle=nil}
LootDrop.MailStacks={}
LootDrop.loaded = false

local tinsert           = table.insert
local zo_strsplit       = zo_strsplit
local ZO_ColorDef       = ZO_ColorDef
local zo_parselink      = ZO_LinkHandler_ParseLink
local zo_min            = zo_min

local Config            = LootDropConfig
local LootDroppable     = LootDroppable
local CBM               = CALLBACK_MANAGER

local LootDropFade      = LootDropFade
local LootDropSlide     = LootDropSlide
local LootDropPop       = LootDropPop

local _

LootDrop_Defaults =
{
	displayduration	= 10,
	experience		= LootDrop_sListBoxExtended,
	coin			= LootDrop_sListBoxExtended,
	loot			= true,
	alliance		= LootDrop_sListBoxStacked,
	battle			= true,
	width			= 220,
	height			= 45,
	padding			= 5,

	sListStyle		= LootDrop_sDefDefault,
	FontSize		= 18,
	rarity			= true,
	MoveUp			= true,
	lootdrop_x		= 0,
	lootdrop_y		= 0,
	lootdrop_lock	= true,
	DbgLog			= false,
	DbgLogTime		= false,
	DbgLogItemStyle	= true,
	DbgLogXp		= true,
	DbgLogGold		= true,
	DbgLogTag		= false,
	Junk			= false,
	mailloot		= true,
	nameloot		= true,
	stackloot		= true,
	styleloot		= false,
	SkillXp			= LootDrop_sListBoxNotStacked, 	--craft
	Guilds			= LootDrop_sListBoxStacked, 	--guild reputation
	Books			= LootDrop_sListBoxNotStacked,	--book knowledge
	Fence			= LootDrop_sListBoxStacked		--Fence skillline
}
-------------------------------------------------------------------------------
LootDrop_sApIcon=[[/lootdrop/textures/ap_up.dds]]
LootDrop_sBtIcon=[[/lootdrop/textures/bt_up.dds]]
LootDrop_sXpIcon=[[/lootdrop/textures/xp_up.dds]]
LootDrop_sSkillXpIcon=[[/lootdrop/textures/skill_up.dds]]


LootDrop_sAnvilXpIcon = "esoui/art/icons/servicemappins/servicepin_smithy.dds"
LootDrop_sWoodXpIcon = "esoui/art/icons/servicemappins/servicepin_woodworking.dds"
LootDrop_sAlchimyXpIcon =  "esoui/art/icons/servicemappins/servicepin_alchemy.dds"
LootDrop_sClothierXpIcon =  "esoui/art/icons/servicemappins/servicepin_outfitter.dds"
LootDrop_sEnchanterXpIcon =  "esoui/art/icons/servicemappins/servicepin_enchanting.dds"
LootDrop_sProvisioningXpIcon =  "esoui/art/icons/servicemappins/servicepin_inn.dds"

LootDrop_sMagesGuildXpIcon = "esoui/art/icons/servicemappins/servicepin_magesguild.dds"
LootDrop_sFighterGuildXpIcon = "esoui/art/icons/servicemappins/servicepin_fightersguild.dds"
LootDrop_sOthersGuildXpIcon = "esoui/art/icons/poi/poi_groupinstance_complete.dds"
LootDrop_sBooksXpIcon = "EsoUI/Art/MainMenu/menuBar_journal_up.dds"
LootDrop_sFenceXpIcon = "/esoui/art/icons/servicemappins/servicepin_fence.dds"

LootDrop_sBgTexture=[[/lootdrop/textures/default_bg.dds]]
LootDrop_sRarityTexture=[[/lootdrop/textures/default_rarity.dds]]
-------------------------------------------------------------------------------
--- Flags for updating UI aspects
local DirtyFlags =
{
    LAYOUT = 1 -- we've added or removed a droppable
}
-------------------------------------------------------------------------------
--- Create our ObjectPool
-- @param ...
function LootDrop:New( ... )
    local result = LootDropPool.New( self )
    result:Initialize( ... )
    return result
end
-------------------------------------------------------------------------------
function LootDrop_LockUnlock()
    LootDropAddon:ToggleLockBiding()
end
-------------------------------------------------------------------------------
function LootDrop_ToggleAutoloot()
    local Setting = GetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_AUTO_LOOT);
	local Autolootinfo = '['..GetTimeString()..']: '
	
	if not LootDropAddon.db.DbgLogTime then
		Autolootinfo=''
	end
    if ("0" ~= Setting) then
        Setting = "0";
        d( Autolootinfo .. 'LootDrop, autoloot OFF');
    else
        Setting = "1";
        d( Autolootinfo .. 'LootDrop, autoloot ON');
    end

    SetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_AUTO_LOOT, Setting, 1);
end
-------------------------------------------------------------------------------
--- I swear I'm going to use this for something
-- @param ...
function LootDrop:Initialize( control )
	self.control = control
	self.control:RegisterForEvent( EVENT_ADD_ON_LOADED, function( ... ) self:OnLoaded( ... ) end )

	LootDropPool.Initialize( self, function() return self:CreateDroppable() end, function( ... ) self:ResetDroppable( ... ) end  )

	self.control:SetHandler( 'OnUpdate',				function( _, ft )		self:OnUpdate( ft )		end )
	self.control:SetHandler( 'OnMoveStop', 				function( )				self:OnMoveStop( )		end )

	self._fadeIn  = LootDropFade:New( 0.0, 1.0, 200 )
	self._fadeOut = LootDropFade:New( 1.0, 0.0, 200 )
	self._slide   = LootDropSlide:New( 200 )
	self._pop     = LootDropPop:New()


	self._coinId = nil
	self._coinLastVal=0

	self._xpId   = nil
	self._xpLastVal=0

	self._xpVetId   = nil
	self._xpVetLastVal=0

	self._apId   = nil
	self._apLastVal=0

	self._btId   = nil
	self._btLastVal=0

	self._skillXpId = nil
	self._skillXpLastVal = 0

	self._skillMageXpId = nil
	self._skillMageXpLastVal = 0

	self._skillFighterXpId = nil
	self._skillFighterXpLastVal = 0

	self._skillUndauntedXpId = nil
	self._skillUndauntedXpLastVal = 0

	self._skillBookXpId = nil

	self._skillFenceXpId = nil
	self._skillFenceXpLastVal = 0

	CBM:RegisterCallback( Config.EVENT_TOGGLE_COIN, function() self:ToggleCoin()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_XP,   function() self:ToggleXP()      end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_LOOT, function() self:ToggleLoot()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_MAIL_LOOT, function() self:ToggleMailLoot()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_AP,   function() self:ToggleAP()      end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_BT,   function() self:ToggleBT()      end )

	CBM:RegisterCallback( Config.EVENT_TOGGLE_STYLE, function() self:ToggleStyle()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_LOCK,  function() self:ToggleLock()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_RARITY,  function() self:ToggleRarity()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_MOVEUP,  function() self:ToggleMoveUp()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_JUNK,  function() self:ToggleJunk()    end )
    CBM:RegisterCallback( Config.EVENT_TOGGLE_DEBUGLOG,  function() self:ToggleDbgLog()    end )
	CBM:RegisterCallback( Config.EVENT_TOGGLE_SKILL_XP,  function() self:ToggleSkillXp()    end )

	CBM:RegisterCallback( Config.EVENT_CHANGE_DIMENSIONS,  function() self:ChangeDimensions()    end )
end
-------------------------------------------------------------------------------
function LootDrop:OnLoaded( event, addon )
	if ( addon ~= 'LootDrop' ) then
		return
	end
	
	self.db     = ZO_SavedVars:NewAccountWide( 'LOOTDROP_DB', 3.0, nil, LootDrop_Defaults )
	self.config = Config:New( self.db )

	self:ToggleCoin()
	self:ToggleXP()
	self:ToggleLoot()
	self:ToggleMailLoot()
	self:ToggleAP()
	self:ToggleBT()

    self:ToggleSkillXp()
	self:ToggleStyle()

	local leftScreen, topScreen, rightScreen, bottomScreen  = GuiRoot:GetScreenRect()
	self.ScreenMaxWidth = rightScreen
	self.ScreenMaxHeight = bottomScreen
	
	self.control:ClearAnchors()
	self.control:SetAnchor(BOTTOMRIGHT, GuiRoot, BOTTOMRIGHT, self.db.lootdrop_x, self.db.lootdrop_y)
	local x=zo_min((self.ScreenMaxWidth-50),(self.db.width+1))
	local y=zo_min((self.ScreenMaxHeight-50),11*(self.db.height + self.db.padding))
	self.control:SetDimensions(x, y)
	self.control:SetDrawLayer(0)
	self.control:SetMovable(not self.db.lootdrop_lock)
	self.control:SetMouseEnabled(not self.db.lootdrop_lock)

	local MoveBG=WINDOW_MANAGER:CreateControl( "LootDropMoveMeBg",  self.control, CT_BACKDROP)  
	LootDropMoveMeBg:SetHidden(self.db.lootdrop_lock)
	LootDropMoveMeBg:SetAnchor(BOTTOMRIGHT, self.control, BOTTOMRIGHT, 0, 0)
	LootDropMoveMeBg:SetAnchorFill(self.control)
	LootDropMoveMeBg:SetCenterColor( 0,0,0,0.4 )
	LootDropMoveMeBg:SetCenterTexture("",8,1,2)
	LootDropMoveMeBg:SetEdgeColor( 0,0,0,0 )
	LootDropMoveMeBg:SetEdgeTexture("",8,1,2)

	self.loaded=true
end
-------------------------------------------------------------------------------
function LootDrop:ChangeDimensions()
	self.control:SetDimensions(zo_min((self.ScreenMaxWidth-50),(self.db.width+1)), zo_min((self.ScreenMaxHeight-50),11*(self.db.height + self.db.padding)))
end
-------------------------------------------------------------------------------
function LootDrop:ToggleMoveUp()
	--nothing to do, function just in case ... ;-)
	self.db.MoveUp = self.db.MoveUp	
end
-------------------------------------------------------------------------------
function LootDrop:ToggleRarity()
	--nothing to do, function just in case ... ;-)
	self.db.rarity = self.db.rarity
end
-------------------------------------------------------------------------------
function LootDrop:ToggleJunk()
	--nothing to do, function just in case ... ;-)
	self.db.Junk = self.db.Junk
end
-------------------------------------------------------------------------------
function LootDrop:ToggleDbgLog()
    --nothing to do, function just in case ... ;-)
    self.db.DbgLog = self.db.DbgLog
	self.db.DbgLogTime = self.db.DbgLogTime
	self.db.DbgLogItemStyle = self.db.DbgLogItemStyle
	self.db.DbgLogXp		= self.db.DbgLogXp
	self.db.DbgLogGold		= self.db.DbgLogGold
	self.db.DbgLogTag		= self.db.DbgLogTag
end
-------------------------------------------------------------------------------
function LootDrop:ToggleStyle()

	self.db.FontSize = self.db.FontSize

	if (self.db.sListStyle==LootDrop_sDefRushmik) then
		LootDrop_sApIcon=[[/lootdrop/textures/ap_rushmik_up.dds]]
		LootDrop_sBtIcon=[[/lootdrop/textures/bt_rushmik_up.dds]]
		LootDrop_sXpIcon=[[/lootdrop/textures/xp_rushmik_up.dds]]
        LootDrop_sSkillXpIcon=[[/lootdrop/textures/skill_rushmik_up.dds]]

		LootDrop_sBgTexture=[[/lootdrop/textures/rushmik_bg.dds]]
		LootDrop_sRarityTexture=[[/lootdrop/textures/rushmik_rarity.dds]]

	elseif (self.db.sListStyle==LootDrop_sDefPawkette) then
		LootDrop_sApIcon=[[/lootdrop/textures/ap_pawkette_up.dds]]
		LootDrop_sBtIcon=[[/lootdrop/textures/bt_pawkette_up.dds]]
		LootDrop_sXpIcon=[[/lootdrop/textures/xp_pawkette_up.dds]]
        LootDrop_sSkillXpIcon=[[/lootdrop/textures/skill_pawkette_up.dds]]

		LootDrop_sBgTexture=[[/lootdrop/textures/default_bg.dds]]
		LootDrop_sRarityTexture=[[/lootdrop/textures/default_rarity.dds]]

	else
	--Default
		LootDrop_sApIcon=[[/lootdrop/textures/ap_up.dds]]
		LootDrop_sBtIcon=[[/lootdrop/textures/bt_up.dds]]
		LootDrop_sXpIcon=[[/lootdrop/textures/xp_up.dds]]
        LootDrop_sSkillXpIcon=[[/lootdrop/textures/skill_up.dds]]

		LootDrop_sBgTexture=[[/lootdrop/textures/default_bg.dds]]
		LootDrop_sRarityTexture=[[/lootdrop/textures/default_rarity.dds]]
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleLockBiding()
    self.db.lootdrop_lock = not self.db.lootdrop_lock
    self:ToggleLock()
end
-------------------------------------------------------------------------------
function LootDrop:ToggleLock()

	self.control:SetMovable(not self.db.lootdrop_lock)
	self.control:SetMouseEnabled(not self.db.lootdrop_lock)
	LootDropMoveMeBg:SetHidden(self.db.lootdrop_lock)

	if (self.db.lootdrop_lock) then
		local leftScreen, topScreen, rightScreen, bottomScreen  = GuiRoot:GetScreenRect()
		local left, top, right, bottom = self.control:GetScreenRect()
		
		self.db.lootdrop_x=math.floor(right-rightScreen)
		self.db.lootdrop_y=math.floor(bottom-bottomScreen)
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleCoin() 
	if ( self.db.coin ~= LootDrop_sListBoxNo ) then
		self.current_money = GetCurrentMoney()
		self.control:RegisterForEvent( EVENT_MONEY_UPDATE, function( _, ... ) self:OnMoneyUpdated( ... )  end )
	else
		self.control:UnregisterForEvent( EVENT_MONEY_UPDATE )
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleXP()
	if ( self.db.experience ~= LootDrop_sListBoxNo ) then
		self.current_Vetxp = GetUnitVeteranPoints("player")
		self.current_xp = GetUnitXP( 'player' )
		self.control:RegisterForEvent( EVENT_EXPERIENCE_UPDATE, function( _, ... ) self:OnXPUpdated( ... )     end )
		self.control:RegisterForEvent( EVENT_VETERAN_POINTS_UPDATE, function( _, ... ) self:OnXPVetUpdated( ... )     end )
	else
		self.control:UnregisterForEvent( EVENT_EXPERIENCE_UPDATE )
		self.control:UnregisterForEvent( EVENT_VETERAN_POINTS_UPDATE )
	end
end

-------------------------------------------------------------------------------
function LootDrop:ToggleSkillXp()
	if (( self.db.SkillXp ~= LootDrop_sListBoxNo ) or ( self.db.Guilds ~= LootDrop_sListBoxNo ) or ( self.db.Books ~= LootDrop_sListBoxNo ) ) then
		self.control:RegisterForEvent( EVENT_SKILL_XP_UPDATE, function( ... ) self:OnSkillXPUpdated( ... )     end )
	else
		self.control:UnregisterForEvent( EVENT_SKILL_XP_UPDATE )
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleLoot()
	if ( self.db.loot ) then
		self.control:RegisterForEvent( EVENT_LOOT_RECEIVED, function( _, ... ) self:OnItemLooted( ... )    end )
		self.control:RegisterForEvent( EVENT_INVENTORY_SINGLE_SLOT_UPDATE, function( ... ) self:OnItemInventoryLooted( ... )    end )
	else
		self.control:UnregisterForEvent( EVENT_LOOT_RECEIVED )
		self.control:UnregisterForEvent( EVENT_INVENTORY_SINGLE_SLOT_UPDATE )
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleMailLoot()
	if ( self.db.mailloot ) and ( self.db.loot )then
		self.control:RegisterForEvent( EVENT_MAIL_TAKE_ATTACHED_ITEM_SUCCESS, function( _, ... ) self:OnMailItemLooted( ... )    end )
		self.control:RegisterForEvent( EVENT_MAIL_READABLE, function( _, ... ) self:OnMailReadable( ... )    end )
	else
		self.control:UnregisterForEvent( EVENT_MAIL_TAKE_ATTACHED_ITEM_SUCCESS )
		self.control:UnregisterForEvent( EVENT_MAIL_READABLE )
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnActiveQuestToolChanged(eventCode, journalIndex, toolIndex)
	local iconFilename, stackCount, isUsable, questToolName = GetQuestToolInfo(journalIndex, toolIndex)
	local color='FFFFFF'

	if ((questToolName ~= nil) and (color ~= nil) and (quantity > 0)) then
		local newDrop, _ = self:Acquire()
		self:LootPrint(newDrop, iconFilename, color, stackCount, questToolName)
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleAP()
	if ( self.db.alliance == LootDrop_sListBoxNo) then
        self.control:UnregisterForEvent( EVENT_ALLIANCE_POINT_UPDATE )
	else
        self.control:RegisterForEvent( EVENT_ALLIANCE_POINT_UPDATE, function( _, ... ) self:OnAPUpdate( ... ) end )
	end
end
-------------------------------------------------------------------------------
function LootDrop:ToggleBT()
	if ( self.db.battle ) then
		self.control:RegisterForEvent( EVENT_BATTLE_TOKEN_UPDATE, function( _, ... ) self:OnBTUpdate( ... ) end )
	else
		self.control:UnregisterForEvent( EVENT_BATTLE_TOKEN_UPDATE )
	end
end
-------------------------------------------------------------------------------
--- Check if any flags are set
-- if no flag is passed will check if any flag is set.
-- @tparam DirtyFlags flag
-- @treturn boolean
function LootDrop:IsDirty( flag )
	if ( not flag ) then return #self.dirty_flags ~= 0 end

	for _,v in pairs( self.dirty_flags ) do
		if ( v == flag ) then
			return true
		end
	end

	return false
end
-------------------------------------------------------------------------------
function LootDrop:OnMoveStop()
	if (not self.loaded) then
		return
	end
	
	local leftScreen, topScreen, rightScreen, bottomScreen  = GuiRoot:GetScreenRect()
	local left, top, right, bottom = self.control:GetScreenRect()
	
	self.db.lootdrop_x=math.floor(right-rightScreen)
	self.db.lootdrop_y=math.floor(bottom-bottomScreen)
end
-------------------------------------------------------------------------------
--- On every consecutive frame
function LootDrop:OnUpdate( frameTime ) 
	if (( not #self._active) or (not self.loaded)) then
		return
	end

	local i = 1
	local entry = nil
	while( i <= #self._active ) do
		entry = self._active[ i ]

		if ( frameTime - entry:GetTimestamp() > self.db.displayduration ) then
			self:Release( entry )
			tinsert( self.dirty_flags, DirtyFlags.LAYOUT )
		else
			i = i + 1
		end
	end

	if ( self:IsDirty( DirtyFlags.LAYOUT ) ) then
		local last_y = 0
		local last_x = 0
		local entry = nil

		if (not self.db.MoveUp) then
			local xdim, ydim = self.control:GetDimensions()
			--y is count from the bottom right, so the top is negative number
			last_y = ( self.db.height + self.db.padding ) - ydim
			last_x = ( self.db.width) - xdim
		end
		
		for i=1,#self._active do
			entry = self._active[ i ]

			if ( not entry:IsVisible() ) then
				entry:Show( last_x, last_y )
			else            
				entry:Move( last_x, last_y )
			end
			
			if (self.db.MoveUp) then
				last_y = last_y - ( self.db.height + self.db.padding )
			else
				--y is count from the bottom right and last y is negative number
				last_y = last_y + ( self.db.height + self.db.padding )
			end
		end
	end

	self.dirty_flags = {}
end
-------------------------------------------------------------------------------
--- Create a new loot droppable
-- @tparam ZO_ObjectPool _ unused
function LootDrop:CreateDroppable()
	return LootDroppable:New( self )
end
-------------------------------------------------------------------------------
--- Reset a loot droppable
-- @tparam LootDroppable droppable 
function LootDrop:ResetDroppable( droppable, key )
	if ( key == self._coinId ) then
		self._coinId = nil
		self._coinLastVal=0
	elseif( key == self._xpId ) then
		self._xpId = nil
		self._xpLastVal=0
	elseif ( key == self._xpVetId ) then
		self._xpVetId=nil
		self._xpVetLastVal=0
	elseif ( key == self._skillXpId ) then
		self._skillXpId = nil
		self._skillXpLastVal = 0
	elseif ( key == self._skillMageXpId ) then
		self._skillMageXpId = nil
		self._skillMageXpLastVal = 0
	elseif ( key == self._skillFighterXpId ) then
		self._skillFighterXpId = nil
		self._skillFighterXpLastVal = 0
	elseif ( key == self._skillUndauntedXpId ) then
		self._skillUndauntedXpId = nil
		self._skillUndauntedXpLastVal = 0
	elseif ( key == self._skillBookXpId ) then
		self._skillBookXpId = nil
	elseif ( key == self._skillFenceXpId ) then
		self._skillFenceXpId = nil
		self._skillFenceXpLastVal = 0
	end

	droppable:Hide()
end
-------------------------------------------------------------------------------
function LootDrop:Acquire()
	local result, key = LootDropPool.Acquire( self )
	result:Prepare(self.db.MoveUp)

	tinsert( self.dirty_flags, DirtyFlags.LAYOUT )

	return result, key
end
-------------------------------------------------------------------------------
function LootDrop:FormatItemName( str )
	return zo_strformat('<<t:1>>', str)
end
-------------------------------------------------------------------------------
function LootDrop:FormatAmount( amount )
	return ZO_CommaDelimitNumber( amount )
end
-------------------------------------------------------------------------------
function LootDrop:ParseLink( link )
	if ( type( link ) ~= 'string' ) then
		return nil, nil
	end

	local text, color = zo_parselink( link )

	if (text=="") then
		text = link 
	end

	if color == "" or color == "0" or color == "1" then
		color = 'FFFFFF'
	end

	return text, color
end
-------------------------------------------------------------------------------
function LootDrop:NewParseLink( NewLink )
	if ( type( NewLink ) ~= 'string' ) then
		return nil, nil
	end

	local Text, LinkStyle = zo_parselink( NewLink )
	Text=zo_strformat("<<x:1>>", Text)

	if (Text=="") then
		Text = NewLink 
	end

	return Text, LinkStyle
end
-------------------------------------------------------------------------------
function LootDrop:ResetCurrentItemBag( )
	LootDrop.CurrentItemBag_ItemStyle = nil
	LootDrop.CurrentItemBag_ItemType = nil
	LootDrop.CurrentItemBag_ItemLink = nil
	LootDrop.CurrentItemBag_ItemQuality = nil
	LootDrop.CurrentItemBag_ItemNb = nil
end
-------------------------------------------------------------------------------
function LootDrop:ResetItemToPrint( )
	LootDrop.ItemToPrint.iconFileName=[[/esoui/art/icons/icon_missing.dds]]
	LootDrop.ItemToPrint.color='FFFFFF'
	LootDrop.ItemToPrint.quantity=0
	LootDrop.ItemToPrint.nb=0
	LootDrop.ItemToPrint.text=''
	LootDrop.ItemToPrint.tag=''
	LootDrop.ItemToPrint.itemstyle=nil
end
-------------------------------------------------------------------------------
--TESO GUILDS TYPES
-- 0 : unknow
-- 1 : Fighter, guerrier, Krieger
-- 2 : Mage, mage, Magier
-- 3 : Undaunted, Indomptable, Unershrockene 
function LootDrop:GetGuildType(skillType,  skillIndex)
	local GuildTypeRet=0
	
	if (skillType ~= SKILL_TYPE_GUILD) then return GuildTypeRet end
	
	local skillName = GetSkillLineInfo(skillType, skillIndex)
	skillName=skillName:lower()

	 if ( (string.find(skillName, "fighter") ~= nil) or (string.find(skillName, "guerrier") ~= nil) or (string.find(skillName, "krieger") ~= nil)) then
		GuildTypeRet=1
	elseif ( (string.find(skillName, "mage") ~= nil ) or (string.find(skillName, "magier") ~= nil )) then
		GuildTypeRet=2
	elseif ( (string.find(skillName, "undaunted") ~= nil) or (string.find(skillName, "indomptable") ~= nil) or (string.find(skillName, "unershrockene") ~= nil) ) then
		GuildTypeRet=3
	end

	return GuildTypeRet
end
-------------------------------------------------------------------------------
function LootDrop:OnSkillXPUpdated( eventCode,  skillType,  skillIndex,  reason,  rank,  previousXP,  currentXP) 

-- SkillType
	-- SKILL_TYPE_ARMOR = 3
	-- SKILL_TYPE_AVA = 6
	-- SKILL_TYPE_CLASS = 1
	-- SKILL_TYPE_GUILD = 5
	-- SKILL_TYPE_NONE = 0
	-- SKILL_TYPE_RACIAL = 7
	-- SKILL_TYPE_TRADESKILL = 8
	-- SKILL_TYPE_WEAPON = 2
	-- SKILL_TYPE_WORLD = 4 
-- reason
	-- PROGRESS_REASON_ACHIEVEMENT = 25
	-- PROGRESS_REASON_ACTION = 13
	-- PROGRESS_REASON_ALLIANCE_POINTS = 33
	-- PROGRESS_REASON_AVA = 15
	-- PROGRESS_REASON_BATTLEGROUND = 6
	-- PROGRESS_REASON_BOOK_COLLECTION_COMPLETE = 12
	-- PROGRESS_REASON_BOSS_KILL = 26
	-- PROGRESS_REASON_COLLECT_BOOK = 11
	-- PROGRESS_REASON_COMMAND = 4
	-- PROGRESS_REASON_COMPLETE_POI = 2
	-- PROGRESS_REASON_DARK_ANCHOR_CLOSED = 28
	-- PROGRESS_REASON_DARK_FISSURE_CLOSED = 29
	-- PROGRESS_REASON_DISCOVER_POI = 3
	-- PROGRESS_REASON_DUNGEON_CHALLENGE = 35
	-- PROGRESS_REASON_EVENT = 27
	-- PROGRESS_REASON_FINESSE = 9
	-- PROGRESS_REASON_GRANT_REPUTATION = 32
	-- PROGRESS_REASON_GUILD_REP = 14
	-- PROGRESS_REASON_JUSTICE_SKILL_EVENT = 36
	-- PROGRESS_REASON_KEEP_REWARD = 5
	-- PROGRESS_REASON_KILL = 0
	-- PROGRESS_REASON_LOCK_PICK = 10
	-- PROGRESS_REASON_MEDAL = 8
	-- PROGRESS_REASON_NONE = -1
	-- PROGRESS_REASON_OTHER = 31
	-- PROGRESS_REASON_OVERLAND_BOSS_KILL = 24
	-- PROGRESS_REASON_PVP_EMPEROR = 34
	-- PROGRESS_REASON_QUEST = 1
	-- PROGRESS_REASON_REWARD = 17
	-- PROGRESS_REASON_SCRIPTED_EVENT = 7
	-- PROGRESS_REASON_SKILL_BOOK = 30
	-- PROGRESS_REASON_TRADESKILL = 16
	-- PROGRESS_REASON_TRADESKILL_ACHIEVEMENT = 18
	-- PROGRESS_REASON_TRADESKILL_CONSUME = 20
	-- PROGRESS_REASON_TRADESKILL_HARVEST = 21
	-- PROGRESS_REASON_TRADESKILL_QUEST = 19
	-- PROGRESS_REASON_TRADESKILL_RECIPE = 22
	-- PROGRESS_REASON_TRADESKILL_TRAIT = 23 
	
--TradeskillType
	-- CRAFTING_TYPE_ALCHEMY = 4
	-- CRAFTING_TYPE_BLACKSMITHING = 1
	-- CRAFTING_TYPE_CLOTHIER = 2
	-- CRAFTING_TYPE_ENCHANTING = 3
	-- CRAFTING_TYPE_INVALID = 0
	-- CRAFTING_TYPE_PROVISIONING = 5
	-- CRAFTING_TYPE_WOODWORKING = 6 

	local bprint=false
	local stacked=false
	local tag=''
	local color='6BB5FF'
	local icon = LootDrop_sSkillXpIcon
	local dropid = self._skillXpId
	local lastval = self._skillXpLastVal
	local GuildType = 0

	--Skill Xp from craft
	if (( skillType == SKILL_TYPE_TRADESKILL ) and (reason == PROGRESS_REASON_TRADESKILL)) then
		bprint=(self.db.SkillXp ~= LootDrop_sListBoxNo)
		stacked=(self.db.SkillXp ~= LootDrop_sListBoxNotStacked)
		tag='CRAFT'

		local CraftType = GetCraftingInteractionType()

		if (CraftType == CRAFTING_TYPE_BLACKSMITHING) then
			icon = LootDrop_sAnvilXpIcon
		elseif (CraftType == CRAFTING_TYPE_CLOTHIER) then
			icon = LootDrop_sClothierXpIcon
		elseif (CraftType == CRAFTING_TYPE_ENCHANTING) then
			icon = LootDrop_sEnchanterXpIcon
		elseif (CraftType == CRAFTING_TYPE_ALCHEMY) then
			icon = LootDrop_sAlchimyXpIcon
		elseif (CraftType == CRAFTING_TYPE_PROVISIONING) then
			icon = LootDrop_sProvisioningXpIcon
		elseif (CraftType == CRAFTING_TYPE_WOODWORKING) then
			icon = LootDrop_sWoodXpIcon
		else -- craft book
			bprint=(self.db.Books ~= LootDrop_sListBoxNo)
			stacked=(self.db.Books ~= LootDrop_sListBoxNotStacked)
			tag='BOOK'
			icon = LootDrop_sBooksXpIcon
			dropid = self._skillBookXpId
			lastval = 0
		end

	--Skill Xp GUILD REPUTATION
	elseif ( skillType == SKILL_TYPE_GUILD ) then
		bprint=(self.db.Guilds ~= LootDrop_sListBoxNo)
		stacked=(self.db.Guilds ~= LootDrop_sListBoxNotStacked)
		tag='GUILD'
		GuildType = self:GetGuildType(skillType,  skillIndex)
		--d('GuildType' .. GuildType)
		if ( GuildType == 1 ) then
			icon = LootDrop_sFighterGuildXpIcon
			dropid = self._skillFighterXpId
			lastval = self._skillFighterXpLastVal
		elseif ( GuildType == 2 ) then
			icon = LootDrop_sMagesGuildXpIcon
			dropid = self._skillMageXpId
			lastval = self._skillMageXpLastVal
		else
			icon = LootDrop_sOthersGuildXpIcon
			dropid = self._skillUndauntedXpId
			lastval = self._skillUndauntedXpLastVal
		end
	--Skill XP from books in library
	elseif (reason == PROGRESS_REASON_SKILL_BOOK) then
		bprint=(self.db.Books ~= LootDrop_sListBoxNo)
		stacked=(self.db.Books ~= LootDrop_sListBoxNotStacked)
		tag='BOOK'
		icon = LootDrop_sBooksXpIcon
		dropid = self._skillBookXpId
		lastval = 0
	-- Fence skilline
	elseif (reason == PROGRESS_REASON_JUSTICE_SKILL_EVENT) then
		bprint=(self.db.Fence ~= LootDrop_sListBoxNo)
		stacked=(self.db.Fence ~= LootDrop_sListBoxNotStacked)
		tag='FENCE'
		icon = LootDrop_sFenceXpIcon
		dropid = self._skillFenceXpId
		lastval = self._skillFenceXpLastVal
	end

	if(bprint) then
		local RealGain = currentXP-previousXP
		local skillName = GetSkillLineInfo(skillType, skillIndex)
		local lastSkillXP, nextSkillXP, currentSkillXP = GetSkillLineXPInfo(skillType, skillIndex)
		local maxedSkill = (nextSkillXP == 0)
		local SkillXp = (currentSkillXP - lastSkillXP)
		local SkillXpTotal = SkillXp
		if (not maxed) then
			SkillXpTotal = nextSkillXP - lastSkillXP
		end

		--DROP LABEL (RealGain value could be stacked in SetSkillNewDrop)
		local pccurrent=0
		if (SkillXpTotal>0) then
			pccurrent = math.floor(100*(SkillXp/SkillXpTotal))
		end
		local pctexts = zo_strformat( '|c736F6E(<<1>>%)|r', pccurrent )
		local finaltext=zo_strformat( '|cFFFFFF[<<1>>]|r', skillName )

		if ((( skillType == SKILL_TYPE_TRADESKILL ) or (skillType == SKILL_TYPE_GUILD) or (( skillType == SKILL_TYPE_WORLD ) and (reason == PROGRESS_REASON_JUSTICE_SKILL_EVENT))) and (tag~='BOOK'))then
			finaltext=pctexts
		end

		--DBG LOG (always not stacked)
		local finaldbgtext=zo_strformat( '|c6BB5FF<<1>>|r  |c736F6E-> <<2>> / <<3>>|r  <<4>> |cFFFFFF[<<t:5>>]|r', self:FormatAmount(RealGain), self:FormatAmount(SkillXp), self:FormatAmount(SkillXpTotal), pctexts, skillName )
				-- d(type(self:FormatAmount(RealGain)) .. ": " .. tostring(self:FormatAmount(RealGain))) --> OK
				-- d(type(self:FormatAmount(SkillXp)) .. ": " .. tostring(self:FormatAmount(SkillXp))) --> OK
				-- d(type(self:FormatAmount(SkillXpTotal)) .. ": " .. tostring(self:FormatAmount(SkillXpTotal))) --> OK
				-- d(type(pctexts) .. ": " .. tostring(pctexts)) --> OK
				-- d(type(skillName) .. ": " .. tostring(skillName)) --> OK
				-- d(type(finaldbgtext) .. ": " .. tostring(finaldbgtext)) --> gives 'rh'
				-- d(self:FormatAmount(RealGain) .. self:FormatAmount(SkillXp) .. self:FormatAmount(SkillXpTotal) .. pctexts .. skillName) -->

		local dropidout, lastvalout = self:SetNewDrop(stacked, dropid, color, icon, lastval, RealGain, finaltext, finaldbgtext, tag)

		--CRAFT
		if (( skillType == SKILL_TYPE_TRADESKILL ) and (reason == PROGRESS_REASON_TRADESKILL)) then
			self._skillXpId = dropidout
			self._skillXpLastVal = lastvalout
		--GUILD REPUTATION
		elseif ( skillType == SKILL_TYPE_GUILD ) then
			if ( GuildType == 1 ) then
				self._skillFighterXpId = dropidout
				self._skillFighterXpLastVal = lastvalout
			elseif ( GuildType == 2 ) then
				self._skillMageXpId = dropidout
				self._skillMageXpLastVal = lastvalout
			else
				self._skillUndauntedXpId = dropidout
				self._skillUndauntedXpLastVal = lastvalout
			end
		--BOOKS
		elseif (reason == PROGRESS_REASON_SKILL_BOOK) then
			self._skillBookXpId = dropidout
		-- FENCE
		elseif (reason == PROGRESS_REASON_JUSTICE_SKILL_EVENT) then
			self._skillFenceXpId = dropidout
			self._skillFenceXpLastVal = lastvalout
		end
	end
end
-------------------------------------------------------------------------------
function LootDrop:SetNewDrop(stacked, dropid, color, icon, lastval, val, label, dbglabel, tag)
	if ((label ~= nil) and (color ~= nil) and (val ~= 0)) then
		local newDrop = nil
        local pop = false

		if (stacked) then
			if ( dropid ) then
				newDrop = self:Get( dropid )
				if ( newDrop ) then
					pop = true
					val = val + lastval
				end
			end
		end

		if ( not newDrop ) then
			newDrop, dropid = self:Acquire()
		end

		lastval = val

		local zo_color = ZO_ColorDef:New( color )
		newDrop:SetTimestamp( GetFrameTimeSeconds() )
		newDrop:SetBackground()

		newDrop:SetLabelSize( self.db.FontSize )
		newDrop:SetRarity( zo_color, self.db.rarity )
		newDrop:SetIcon( icon )
		local slabel = zo_strformat( ' |c6BB5FF<<1>>|r  <<t:2>>', self:FormatAmount(val), label)
		newDrop:SetLabel(slabel)
		if ( dbglabel == nil ) then dbglabel = slabel end
		--d(dbglabel)--> label ok
		if (self.db.DbgLogXp) then
			self:dbgDrop(icon, color, 1, dbglabel, tag)
		end
		
		if ( pop ) then
			local anim = self._pop:Apply( newDrop.control )
			anim:Forward()
		end
	end

	return dropid, lastval
end
-------------------------------------------------------------------------------
function LootDrop:GetStyleColoredString( itemStyle )
	local stylestring = ''
	local StyleColoredString =nil
	--d('COLORED')
	--style info
	if ((itemStyle ~= ITEMSTYLE_NONE) and (itemStyle ~= nil)) then
		stylestring = zo_strformat("<<1>>", GetString("SI_ITEMSTYLE", itemStyle))
		--d(stylestring)
		local color = "FFFFFF"

		if itemStyle >= ITEMSTYLE_RACIAL_BRETON and itemStyle <= ITEMSTYLE_RACIAL_KHAJIIT then
			color = GetItemQualityColor(ITEM_QUALITY_ARCANE)
		elseif itemStyle == ITEMSTYLE_RACIAL_IMPERIAL or itemStyle == ITEMSTYLE_AREA_DWEMER then
			color = GetItemQualityColor(ITEM_QUALITY_LEGENDARY)
		else
			color = GetItemQualityColor(ITEM_QUALITY_ARTIFACT)
		end
		StyleColoredString = ("|c%s [%s]|r"):format (color:ToHex(), stylestring)
	end

	--d(StyleColoredString)
	return StyleColoredString
end
-------------------------------------------------------------------------------
function LootDrop:OnItemInventoryLooted( eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason)

	if (updateReason == INVENTORY_UPDATE_REASON_DURABILITY_CHANGE or (bagId == BAG_WORN))then
		return
	end

	local icon,  stack,  sellPrice,  meetsUsageRequirement,  locked,  equipType,  itemStyle,  quality = GetItemInfo( bagId, slotId)

	self.CurrentItemBag_ItemType	= GetItemType(bagId, slotId)
	if (( self.CurrentItemBag_ItemType == ITEMTYPE_GLYPH_ARMOR ) or ( self.CurrentItemBag_ItemType ==  ITEMTYPE_GLYPH_JEWELRY ) or ( self.CurrentItemBag_ItemType ==  ITEMTYPE_GLYPH_WEAPON )) then
		--ITEMTYPE_STYLE_MATERIAL ?
		self.CurrentItemBag_ItemStyle	= nil
	else
		self.CurrentItemBag_ItemStyle	= self:GetStyleColoredString( itemStyle )
	end

	self.CurrentItemBag_ItemLink	= GetItemLink(bagId, slotId, LINK_STYLE_DEFAULT)
	self.CurrentItemBag_ItemQuality	= quality
	self.CurrentItemBag_ItemNb		= GetItemTotalCount(bagId,slotId)

	if ((self.db.Junk) and (self.CurrentItemBag_ItemType == ITEMTYPE_TRASH)) then
		SetItemIsJunk(bagId, slotId, true)
	end

	--for mail loot
	if ( self.db.mailloot ) then
		--d(self.MailStacks)
		local bagname = GetItemLinkName(self.CurrentItemBag_ItemLink)
		for mailitemindex, item in pairs(self.MailStacks) do
			local mailitemname = GetItemLinkName(item.itemlink)
			if (bagname == mailitemname) then
				item.quality = self.CurrentItemBag_ItemQuality
				item.nbtotal = self.CurrentItemBag_ItemNb
				item.itemstyle = self.CurrentItemBag_ItemStyle
				--found, so get out
				break
			end
		end
	end
end
-------------------------------------------------------------------------------
--- Called when you loot an Item
-- @tparam string itemName (could be a string or an itemlink)
-- @tparam number quantity 
-- @tparam boolean mine
function LootDrop:OnItemLooted( receivedBy, itemName, quantity, itemSound, lootType, mine )
	if ( not mine ) then
		return
	end

	local icon, sellPrice, meetsUsageRequirement, equipType, itemStyle = GetItemLinkInfo( itemName )
	local itemlink=itemName
	local color = 'FFFFFF'
	self.ItemToPrint.tag = 'LOOT'

	local bagname = GetItemLinkName(self.CurrentItemBag_ItemLink)
	local itemlinkname = GetItemLinkName(itemName)

	if ((bagname==itemName) or (self.CurrentItemBag_ItemLink==itemName) or (itemlinkname==bagname)) then 
		if ((self.CurrentItemBag_ItemLink~=nil) and (self.CurrentItemBag_ItemQuality~=nil)) then
			local iconBag, sellPriceBag, meetsUsageRequirementBag, equipTypeBag, itemStyleBag = GetItemLinkInfo( self.CurrentItemBag_ItemLink )
			color = GetItemQualityColor(self.CurrentItemBag_ItemQuality):ToHex()

			if ((self.db.Junk) and (self.CurrentItemBag_ItemType == ITEMTYPE_TRASH)) then
				self.ItemToPrint.tag = 'JUNK'
			else
				self.ItemToPrint.tag = 'INV'
			end

			icon=iconBag
			self.ItemToPrint.nb=self.CurrentItemBag_ItemNb
			self.ItemToPrint.itemstyle=self.CurrentItemBag_ItemStyle
			itemlink=self.CurrentItemBag_ItemLink
			self.ResetCurrentItemBag()
		end
	end

	--Quests Tool
	if(lootType == LOOT_TYPE_QUEST_ITEM) then
		if ( (not icon) or (icon == '') or (icon == [[/esoui/art/icons/icon_missing.dds]])) then
			self.ItemToPrint.tag = 'QUEST'
			icon = [[/esoui/art/inventory/inventory_tabicon_quest_down.dds]]
		end
	end

	--test if invalid itemName ==> function returns nil, nil
	if ((itemlink ~= nil) and (color ~= nil) and (quantity > 0)) then
		local newDrop, _ = self:Acquire()
		self:LootPrint(newDrop, icon, color, quantity, itemlink)
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnMailReadable(mailId)

	self.MailStacks = {}

	local numAttachments, attachedMoney, codAmount = GetMailAttachmentInfo( mailId )

	local attachIndex = 1
	while( attachIndex <= numAttachments ) do
		self.MailStacks[attachIndex] = {}
		local icon, stack, creatorName = GetAttachedItemInfo( mailId,  attachIndex)
		local mailitemlink = GetAttachedItemLink( mailId,  attachIndex, LINK_STYLE_DEFAULT)
		self.MailStacks[attachIndex].stack = stack
		self.MailStacks[attachIndex].icon = icon
		self.MailStacks[attachIndex].itemlink=mailitemlink

		attachIndex = attachIndex + 1
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnMailItemLooted(mailId)

	--for mail loot
	--d(self.MailStacks)
	for mailitemindex, item in pairs(self.MailStacks) do
		if (item~=nil) then
			local color = 'FFFFFF'
			if(item.quality~=nil) then
				color=GetItemQualityColor(item.quality):ToHex()
			end
			
			self.ItemToPrint.nb = 1
			
			if (item.nbtotal~=nil) then
				self.ItemToPrint.nb = item.nbtotal
			end
			
			self.ItemToPrint.tag = 'MAIL'
			local itemlink =item.itemlink
			local icon = item.icon
			local stackmail = 1
			
			if (item.stack~=nil) then
				stackmail=item.stack
			end
			self.ItemToPrint.itemstyle=item.itemstyle

			if ((itemlink ~= nil) and (color ~= nil)) then
					local newDrop, _ = self:Acquire()
					self:LootPrint(newDrop, icon, color, stackmail, itemlink)
			end
		end
	end

	self.MailStacks = {}
	self.ResetCurrentItemBag()
end
-------------------------------------------------------------------------------
function LootDrop:LootPrint(newDrop, icon, c, quantity, text)

	if ( not icon or icon == '' )then
		icon = [[/esoui/art/icons/icon_missing.dds]]    
	end

	if (quantity > 0) then
		local color = ZO_ColorDef:New( c )
		text = self:FormatItemName( text )
		newDrop:SetTimestamp( GetFrameTimeSeconds() )
		newDrop:SetBackground()
		newDrop:SetLabelSize( self.db.FontSize )
		newDrop:SetRarity( color , self.db.rarity)
		newDrop:SetIcon( icon )

		local label = zo_strformat( ' <<1[//$dx]>> <<t:2>>', quantity, text )
		local tag = self.ItemToPrint.tag

		if ( not self.db.nameloot ) and ((tag == 'INV') or (tag == 'JUNK') or (tag == 'MAIL')) then 
			label = zo_strformat( ' <<1[//$dx]>>', quantity )
		end

		local invstack = ''
		local style = ''

		if ((tag=='INV') or (tag=='JUNK') or (tag=='MAIL')) then
			if (self.db.stackloot) then
				invstack=zo_strformat( ' |c736F6E(<<1>>)|r', self.ItemToPrint.nb)
			end
			if (self.db.styleloot) then
				if (self.ItemToPrint.itemstyle~=nil) then
					style=self.ItemToPrint.itemstyle
				end
			end
		end

		label = label .. invstack .. style
		newDrop:SetLabel( label )

		local anim = self._pop:Apply( newDrop.control )
		anim:Forward()

        self:dbgDrop(icon, c, quantity, text, tag)
    end

	self.ResetItemToPrint()
end
-------------------------------------------------------------------------------
--- Called when the amount of money you have changes
-- @tparam number money 
function LootDrop:OnMoneyUpdated( money )
	if ( self.current_money == money ) then
		return
	end

	local difference = money - self.current_money
	self.current_money = money
    local RealDiff=difference

	local pop = false

	if (difference ~= 0) then
		local newDrop = nil

		if ( self._coinId ) then
			newDrop = self:Get( self._coinId )
			if ( newDrop ) then
				pop = true
				difference = difference + self._coinLastVal
			end
		end

		if ( not newDrop ) then
			newDrop, self._coinId = self:Acquire()
		end

		self._coinLastVal = difference

		local c = 'FFFF66'

		newDrop:SetTimestamp( GetFrameTimeSeconds() )
		newDrop:SetBackground()
		newDrop:SetLabelSize( self.db.FontSize )
		newDrop:SetRarity( ZO_ColorDef:New( c ), self.db.rarity )
		newDrop:SetIcon( [[/esoui/art/icons/item_generic_coinbag.dds]] )

		if (self.db.coin == LootDrop_sListBoxExtended) then
			newDrop:SetLabel(zo_strformat('<<1>>  |c736F6E-> <<2>>|r', self:FormatAmount(difference), self:FormatAmount(self.current_money)))
		else
			newDrop:SetLabel( self:FormatAmount(difference) )
		end

		if (self.db.DbgLogGold) then
			local text=zo_strformat('<<1>>  |c736F6E-> <<2>>|r', self:FormatAmount(RealDiff), self:FormatAmount(self.current_money))
			self:dbgDrop([[/esoui/art/icons/item_generic_coinbag.dds]], c, 1, text, '')
		end

		if ( pop ) then
			local anim = self._pop:Apply( newDrop.control )
			anim:Forward()
		end
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnXPUpdated( tag, exp, maxExp, reason )
	if ( tag ~= 'player' ) then
		return
	end

	local xp = exp
	if (maxExp > 0) then
		xp = zo_min( exp, maxExp )
	end

	--if maxExp == 0 then max standard rank is reached
	if ( self.current_xp == xp ) or (maxExp == 0) then
		return 
	end

	local gain = xp - self.current_xp

	if (exp < maxExp) then
		self.current_xp = xp
	else
		self.current_xp= 0
	end

	if ( gain <= 0 ) then
		return
    end

    local RealDiff=gain

	local pop = false

	local newDrop = nil
	if ( self._xpId ) then
		newDrop = self:Get( self._xpId )

		if ( newDrop ) then
			pop = true
			gain = gain + self._xpLastVal
		end
	end

	if ( not newDrop ) then
		newDrop, self._xpId = self:Acquire()
	end

	self._xpLastVal = gain

	local c = 'FFFFFF'
	if (self.db.sListStyle==LootDrop_sDefPawkette) then
		c = '00FF00'
	end


	newDrop:SetTimestamp( GetFrameTimeSeconds() )
	newDrop:SetBackground()
	newDrop:SetLabelSize( self.db.FontSize )
	newDrop:SetRarity( ZO_ColorDef:New( c ), self.db.rarity )
	newDrop:SetIcon( LootDrop_sXpIcon )

	if (self.db.experience == LootDrop_sListBoxExtended) then
		local pcgain = 0
		local pccurrent=100
	
		if (maxExp>0) then
			pccurrent = math.floor(100*(xp/maxExp))
			pcgain = (math.floor(100000 * (gain/maxExp)) / 1000)
		end
		local pctexts = zo_strformat( '|c736F6E(+<<1>>% -> <<2>>%)|r', pcgain, pccurrent )
		newDrop:SetLabel( zo_strformat('<<1>> <<2>>', self:FormatAmount(gain), pctexts))
	else
		newDrop:SetLabel( self:FormatAmount(gain) )
	end

	if (self.db.DbgLogXp) then
		local text=zo_strformat('<<1>>  |c736F6E-> <<2>> / <<3>>|r', self:FormatAmount(RealDiff), self:FormatAmount(xp), self:FormatAmount(maxExp))
		self:dbgDrop(LootDrop_sXpIcon, c, 1, text, 'Xp')
	end

	if ( pop ) then
		local anim = self._pop:Apply( newDrop.control )
		anim:Forward()
	end
end

-------------------------------------------------------------------------------
function LootDrop:OnXPVetUpdated( tag, exp, maxExp, reason )
	if ( tag ~= 'player' ) then
		return
	end
	--LootDrop_MaxVetRank
	local MaxVetRankReached = (GetUnitVeteranRank(tag) >= LootDrop_MaxVetRank)

	local xp = exp
	if ((maxExp > 0) and (not MaxVetRankReached)) then
		xp = zo_min( exp, maxExp )
	end

	if ( self.current_Vetxp == xp ) then
		return 
	end

	local gain = xp - self.current_Vetxp

	if ((exp < maxExp) or (MaxVetRankReached)) then
		self.current_Vetxp = xp
	else
		self.current_Vetxp= 0
	end
	
	if (MaxVetRankReached) then
		xp = GetPlayerChampionXP()
		maxExp = GetChampionXPInRank(GetPlayerChampionPointsEarned()) or GetChampionXPInRank(0)
	end

	if ( gain <= 0 ) then
		return
	end

    local RealDiff=gain
	local pop = false
	local newDrop = nil

    if ( self._xpVetId ) then
        newDrop = self:Get( self._xpVetId )

		if ( newDrop ) then
			pop = true
			gain = gain + self._xpVetLastVal
		end
	end

	if ( not newDrop ) then
		newDrop, self._xpVetId = self:Acquire()
	end

	self._xpVetLastVal= gain

	local c = 'FFFFFF'
	if (self.db.sListStyle==LootDrop_sDefPawkette) then
		c = '00FF00'
	end


	newDrop:SetTimestamp( GetFrameTimeSeconds() )
	newDrop:SetBackground()
	newDrop:SetLabelSize( self.db.FontSize )
	newDrop:SetRarity( ZO_ColorDef:New( c ), self.db.rarity )
	newDrop:SetIcon( LootDrop_sXpIcon )
	
	if (self.db.experience == LootDrop_sListBoxExtended) then
		local pcgain = 0
		local pccurrent=100
	
		if (maxExp>0) then
			pccurrent = math.floor(100*(xp/maxExp))
			pcgain = (math.floor(100000 * (gain/maxExp)) / 1000)
		end
		local pctexts = zo_strformat( '|c736F6E(+<<1>>% -> <<2>>%)|r', pcgain, pccurrent )

		newDrop:SetLabel( zo_strformat('<<1>> <<2>>', self:FormatAmount(gain), pctexts ) )
	else
		newDrop:SetLabel(self:FormatAmount(gain) )
	end

	if (self.db.DbgLogXp) then
		local text = zo_strformat('<<1>> |c736F6E -> <<2>> / <<3>>|r', self:FormatAmount(RealDiff), self:FormatAmount(xp), self:FormatAmount(maxExp))
		self:dbgDrop(LootDrop_sXpIcon, c, 1, text, 'XpVet')
	end

	if ( pop ) then
		local anim = self._pop:Apply( newDrop.control )
		anim:Forward()
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnAPUpdate( _, _, difference )

	if ((difference <= 0) or (self.db.alliance == LootDrop_sListBoxNo))then return end
	
	local pop = false
	local newDrop = nil
	local RealDiff=difference

	if (self.db.alliance ~= LootDrop_sListBoxNotStacked) then
		if ( self._apId ) then
			newDrop = self:Get( self._apId )
			if ( newDrop ) then
				pop = true
				difference = difference + self._apLastVal
			end
		end
	end

	if ( not newDrop ) then
		newDrop, self._apId = self:Acquire()
	end

	self._apLastVal=difference

	local c = 'C5FFC0'
	if (self.db.sListStyle==LootDrop_sDefPawkette) then
		c = '0000FF'
	end

	newDrop:SetTimestamp( GetFrameTimeSeconds() ) 
	newDrop:SetBackground()
	newDrop:SetLabelSize( self.db.FontSize )
	newDrop:SetRarity( ZO_ColorDef:New( c ) , self.db.rarity)
	newDrop:SetIcon( LootDrop_sApIcon )
	newDrop:SetLabel( self:FormatAmount(difference) )

	if (self.db.DbgLogXp) then
		self:dbgDrop(LootDrop_sApIcon, c, 1, self:FormatAmount(RealDiff), 'AP')
	end

	if ( pop ) then
		local anim = self._pop:Apply( newDrop.control )
		anim:Forward()
	end
end
-------------------------------------------------------------------------------
function LootDrop:OnBTUpdate( _, _, difference )
	if ((difference <= 0) or (self.db.alliance == LootDrop_sListBoxNo))then return end

	local pop = false
	local newDrop = nil

    local RealDiff=difference

	if ( self._btId ) then
		newDrop = self:Get( self._btId )

		if ( newDrop ) then
			pop = true
			difference = difference + self._btLastVal
		end
	end

	if ( not newDrop ) then
		newDrop, self._btId = self:Acquire()
	end

	self._btLastVal = difference

    local c = 'FFC3BD'
	if (self.db.sListStyle==LootDrop_sDefPawkette) then
        c = 'FF0000'
	end

	newDrop:SetTimestamp( GetFrameTimeSeconds() )
	newDrop:SetBackground()
	newDrop:SetLabelSize( self.db.FontSize )
	newDrop:SetRarity( ZO_ColorDef:New( c ) , self.db.rarity)
	newDrop:SetIcon( LootDrop_sBtIcon )
	newDrop:SetLabel( self:FormatAmount(difference) )

	if (self.db.DbgLogXp) then
		self:dbgDrop(LootDrop_sBtIcon, c, 1, self:FormatAmount(RealDiff), 'BT')
	end

	if ( pop ) then
		local anim = self._pop:Apply( newDrop.control )
		anim:Forward()
	end
end
-------------------------------------------------------------------------------
--- Getter for the control xml element
-- @treturn table 
function LootDrop:GetControl()
	return self.control
end
-------------------------------------------------------------------------------
function LootDrop_Initialized( self )
    LootDropAddon = LootDrop:New( self )
end
-------------------------------------------------------------------------------
function LootDrop:dbgDrop(iconFilename, c, quantity, text, tag)

	if (not self.db.DbgLog) then return end

	if ( not iconFilename or iconFilename == '' )then
		iconFilename = [[/esoui/art/icons/icon_missing.dds]]
	end

	if (quantity ~= 0) then
		local color = ZO_ColorDef:New( c )
		local dbgText1='[' .. GetTimeString() .. ']:'

		if not self.db.DbgLogTime then
			dbgText1 = ''
		end
		
		local icon = CreateIcon(iconFilename)

		local dbgText2=zo_strformat( '<<1[//$dx]>> <<2>>', quantity, text)
		-- d(dbgText2) --> label OK
		-- d(type(text) .. ": " .. tostring(text)) --> gives 'rh'
		-- d(type(dbgText2) .. ": " .. tostring(dbgText2)) --> have 'rh'
		-- d(type(quantity) .. ": " .. tostring(quantity))  --> Quantity OK, it's a number 
		-- d(type(icon) .. ": " ..  tostring(icon)) --> icon OK
		
		if ((tag=='INV') or (tag=='JUNK') or (tag=='MAIL')) then
			dbgText2=zo_strformat( '<<1[//$dx]>> <<2>> |c736F6E(<<3>>)|r', quantity, text, self.ItemToPrint.nb)
			if ((self.db.DbgLogItemStyle) and (self.ItemToPrint.itemstyle~=nil)) then
				dbgText2=dbgText2..self.ItemToPrint.itemstyle
			end
		end

		local dbgText3='.'
		if ((tag~='') and  (self.db.DbgLogTag)) then dbgText3=' (' .. tag .. ').' end

		local finaldbgtext=zo_strformat( '<<1>> <<2>> <<3>><<4>>', dbgText1, icon, dbgText2, dbgText3)
		d(finaldbgtext)
	end
end
--------------------------------------------------------------------------------------------------------------
--TOOLS
--------------------------------------------------------------------------------------------------------------
local OkdebugLootDrop=false
--var always ~=nil
function dbgLoot(...)
	if OkdebugLootDrop then
		d('['..GetTimeString()..']: ' .. tostring(...))
	end
end
-------------------------------------------------------------------------------
function CreateIcon(filename, width, height)
	return zo_iconFormat(filename, width or 16, height or 16)
end
-------------------------------------------------------------------------------